import json
import os
import sys
from subprocess import PIPE, Popen
from urllib.parse import urlsplit

from packaging.version import InvalidVersion, Version
from scrapy.utils.misc import load_object
from twisted.web import resource

from scrapyd.config import Config
from scrapyd.sqlite import JsonSqliteDict


class JsonResource(resource.Resource):

    json_encoder = json.JSONEncoder()

    def render(self, txrequest):
        r = resource.Resource.render(self, txrequest)
        return self.render_object(r, txrequest)

    def render_object(self, obj, txrequest):
        r = self.json_encoder.encode(obj) + "\n"
        txrequest.setHeader('Content-Type', 'application/json')
        txrequest.setHeader('Access-Control-Allow-Origin', '*')
        txrequest.setHeader('Access-Control-Allow-Methods', 'GET, POST, PATCH, PUT, DELETE')
        txrequest.setHeader('Access-Control-Allow-Headers', ' X-Requested-With')
        txrequest.setHeader('Content-Length', str(len(r)))
        return r


class UtilsCache:
    # array of project name that need to be invalided
    invalid_cached_projects = []

    def __init__(self):
        self.cache_manager = JsonSqliteDict(table="utils_cache_manager")

    # Invalid the spider's list's cache of a given project (by name)
    @staticmethod
    def invalid_cache(project):
        UtilsCache.invalid_cached_projects.append(project)

    def __getitem__(self, key):
        for p in UtilsCache.invalid_cached_projects:
            if p in self.cache_manager:
                del self.cache_manager[p]
        UtilsCache.invalid_cached_projects[:] = []
        return self.cache_manager[key]

    def __setitem__(self, key, value):
        self.cache_manager[key] = value


def get_spider_queues(config):
    """Return a dict of Spider Queues keyed by project name"""
    spiderqueue = load_object(config.get('spiderqueue', 'scrapyd.spiderqueue.SqliteSpiderQueue'))
    return {project: spiderqueue(config, project) for project in get_project_list(config)}


def sqlite_connection_string(config, database):
    dbs_dir = config.get('dbs_dir', 'dbs')
    if dbs_dir == ':memory:' or urlsplit(dbs_dir).scheme:
        return dbs_dir
    if not os.path.exists(dbs_dir):
        os.makedirs(dbs_dir)
    return os.path.join(dbs_dir, f'{database}.db')


def get_project_list(config):
    """Get list of projects by inspecting the eggs storage and the ones defined in
    the scrapyd.conf [settings] section
    """
    eggstorage = config.get('eggstorage', 'scrapyd.eggstorage.FilesystemEggStorage')
    eggstoragecls = load_object(eggstorage)
    eggstorage = eggstoragecls(config)
    projects = eggstorage.list_projects()
    projects.extend(x[0] for x in config.items('settings', default=[]))
    return projects


def native_stringify_dict(dct_or_tuples, encoding='utf-8', keys_only=True):
    """Return a (new) dict with unicode keys (and values when "keys_only" is
    False) of the given dict converted to strings. `dct_or_tuples` can be a
    dict or a list of tuples, like any dict constructor supports.
    """
    d = {}
    for k, v in dct_or_tuples.items():
        k = _to_native_str(k, encoding)
        if not keys_only:
            if isinstance(v, dict):
                v = native_stringify_dict(v, encoding=encoding, keys_only=keys_only)
            elif isinstance(v, list):
                v = [_to_native_str(e, encoding) for e in v]
            else:
                v = _to_native_str(v, encoding)
        d[k] = v
    return d


def get_crawl_args(message):
    """Return the command-line arguments to use for the scrapy crawl process
    that will be started for this message
    """
    msg = message.copy()
    args = [_to_native_str(msg['_spider'])]
    del msg['_project'], msg['_spider']
    settings = msg.pop('settings', {})
    for k, v in native_stringify_dict(msg, keys_only=False).items():
        args += ['-a']
        args += ['%s=%s' % (k, v)]
    for k, v in native_stringify_dict(settings, keys_only=False).items():
        args += ['-s']
        args += ['%s=%s' % (k, v)]
    return args


def get_spider_list(project, runner=None, pythonpath=None, version=''):
    """Return the spider list from the given project, using the given runner"""
    if "cache" not in get_spider_list.__dict__:
        get_spider_list.cache = UtilsCache()
    try:
        return get_spider_list.cache[project][version]
    except KeyError:
        pass
    if runner is None:
        runner = Config().get('runner')
    env = os.environ.copy()
    env['PYTHONIOENCODING'] = 'UTF-8'
    env['SCRAPY_PROJECT'] = project
    if pythonpath:
        env['PYTHONPATH'] = pythonpath
    if version:
        env['SCRAPYD_EGG_VERSION'] = version
    pargs = [sys.executable, '-m', runner, 'list', '-s', 'LOG_STDOUT=0']
    proc = Popen(pargs, stdout=PIPE, stderr=PIPE, env=env)
    out, err = proc.communicate()
    if proc.returncode:
        msg = err or out or ''
        msg = msg.decode('utf8')
        raise RuntimeError(msg)
    # FIXME: can we reliably decode as UTF-8?
    # scrapy list does `print(list)`
    tmp = out.decode('utf-8').splitlines()
    try:
        project_cache = get_spider_list.cache[project]
        project_cache[version] = tmp
    except KeyError:
        project_cache = {version: tmp}
    get_spider_list.cache[project] = project_cache
    return tmp


def _to_native_str(text, encoding='utf-8', errors='strict'):
    if isinstance(text, str):
        return text
    if not isinstance(text, (bytes, str)):
        raise TypeError('_to_native_str must receive a bytes, str or unicode '
                        'object, got %s' % type(text).__name__)

    return text.decode(encoding, errors)


def sorted_versions(versions):
    try:
        return sorted(versions, key=Version)
    except InvalidVersion:
        return sorted(versions)



from integration_tests import req


def test_root():
    response = req("get", "/")

    assert '"/jobs"' in response.text
    assert '"/logs/"' in response.text


def test_paths():
    for page in ("/jobs", "/logs"):
        req("get", page)


def test_base_path():
    response = req("get", "/", headers={"X-Forwarded-Prefix": "/base/path"})

    assert '"/base/path/jobs"' in response.text
    assert '"/base/path/logs/"' in response.text



import os
import sys
from contextlib import contextmanager

from scrapy.utils.misc import load_object

from scrapyd import Config
from scrapyd.eggutils import activate_egg


@contextmanager
def project_environment(project):
    config = Config()
    eggstorage_path = config.get(
        'eggstorage', 'scrapyd.eggstorage.FilesystemEggStorage'
    )
    eggstorage_cls = load_object(eggstorage_path)
    eggstorage = eggstorage_cls(config)

    eggversion = os.environ.get('SCRAPYD_EGG_VERSION', None)
    version, eggfile = eggstorage.get(project, eggversion)
    if eggfile:
        activate_egg(eggfile.name)
        eggfile.close()

    assert 'scrapy.conf' not in sys.modules, "Scrapy settings already loaded"
    yield


def main():
    project = os.environ['SCRAPY_PROJECT']
    with project_environment(project):
        from scrapy.cmdline import execute
        execute()


if __name__ == '__main__':
    main()



from io import BytesIO
from unittest.mock import patch

from twisted.trial import unittest
from zope.interface import implementer
from zope.interface.verify import verifyObject

from scrapyd.app import application
from scrapyd.config import Config
from scrapyd.eggstorage import FilesystemEggStorage
from scrapyd.interfaces import IEggStorage


@implementer(IEggStorage)
class SomeFakeEggStorage:
    def __init__(self, config):
        self.config = config

    def put(self, eggfile, project, version):
        pass

    def get(self, project, version=None):
        pass

    def list(self, project):
        pass

    def list_projects(self):
        return ['hello_world']

    def delete(self, project, version=None):
        pass


class TestConfigureEggStorage(unittest.TestCase):
    def test_egg_config_application(self):
        config = Config()
        eggstore = 'scrapyd.tests.test_eggstorage.SomeFakeEggStorage'
        config.cp.set('scrapyd', 'eggstorage', eggstore)
        app = application(config)
        app_eggstorage = app.getComponent(IEggStorage)

        self.assertIsInstance(app_eggstorage, SomeFakeEggStorage)
        self.assertEqual(app_eggstorage.list_projects(), ['hello_world'])


class EggStorageTest(unittest.TestCase):

    def setUp(self):
        d = self.mktemp()
        config = Config(values={'eggs_dir': d})
        self.eggst = FilesystemEggStorage(config)

    def test_interface(self):
        verifyObject(IEggStorage, self.eggst)

    @patch('scrapyd.eggstorage.glob', new=lambda x: ['ddd', 'abc', 'bcaa'])
    def test_list_hashes(self):
        versions = self.eggst.list('any')

        self.assertEqual(versions, ['abc', 'bcaa', 'ddd'])

    @patch('scrapyd.eggstorage.glob', new=lambda x: ['9', '2', '200', '3', '4'])
    def test_list_semantic_versions(self):
        versions = self.eggst.list('any')

        self.assertEqual(versions, ['2', '3', '4', '9', '200'])

    def test_put_get_list_delete(self):
        self.eggst.put(BytesIO(b"egg01"), 'mybot', '01')
        self.eggst.put(BytesIO(b"egg03"), 'mybot', '03/ver')
        self.eggst.put(BytesIO(b"egg02"), 'mybot', '02_my branch')

        self.assertEqual(self.eggst.list('mybot'), [
            '01',
            '02_my_branch',
            '03_ver'
        ])
        self.assertEqual(self.eggst.list('mybot2'), [])

        v, f = self.eggst.get('mybot')
        self.assertEqual(v, "03_ver")
        self.assertEqual(f.read(), b"egg03")
        f.close()

        v, f = self.eggst.get('mybot', '02_my branch')
        self.assertEqual(v, "02_my branch")
        self.assertEqual(f.read(), b"egg02")
        f.close()

        v, f = self.eggst.get('mybot', '02_my_branch')
        self.assertEqual(v, "02_my_branch")
        self.assertEqual(f.read(), b"egg02")
        f.close()

        self.eggst.delete('mybot', '02_my branch')
        self.assertEqual(self.eggst.list('mybot'), ['01', '03_ver'])

        self.eggst.delete('mybot', '03_ver')
        self.assertEqual(self.eggst.list('mybot'), ['01'])

        self.eggst.delete('mybot')
        self.assertEqual(self.eggst.list('mybot'), [])



import socket
from datetime import datetime, timedelta
from urllib.parse import urlparse

from scrapy.utils.misc import load_object
from twisted.application.service import IServiceCollection
from twisted.web import resource, static

from scrapyd.interfaces import IEggStorage, IPoller, ISpiderScheduler
from scrapyd.jobstorage import job_items_url, job_log_url


class PrefixHeaderMixin:
    def get_base_path(self, txrequest):
        return txrequest.getHeader(self.prefix_header) or ''


class Root(resource.Resource):

    def __init__(self, config, app):
        resource.Resource.__init__(self)
        self.debug = config.getboolean('debug', False)
        self.runner = config.get('runner')
        self.prefix_header = config.get('prefix_header')
        logsdir = config.get('logs_dir')
        itemsdir = config.get('items_dir')
        self.local_items = itemsdir and (urlparse(itemsdir).scheme.lower() in ['', 'file'])
        self.app = app
        self.nodename = config.get('node_name', socket.gethostname())
        self.putChild(b'', Home(self, self.local_items))
        if logsdir:
            self.putChild(b'logs', static.File(logsdir.encode('ascii', 'ignore'), 'text/plain'))
        if self.local_items:
            self.putChild(b'items', static.File(itemsdir, 'text/plain'))
        self.putChild(b'jobs', Jobs(self, self.local_items))
        services = config.items('services', ())
        for servName, servClsName in services:
            servCls = load_object(servClsName)
            self.putChild(servName.encode('utf-8'), servCls(self))
        self.update_projects()

    def update_projects(self):
        self.poller.update_projects()
        self.scheduler.update_projects()

    @property
    def launcher(self):
        app = IServiceCollection(self.app, self.app)
        return app.getServiceNamed('launcher')

    @property
    def scheduler(self):
        return self.app.getComponent(ISpiderScheduler)

    @property
    def eggstorage(self):
        return self.app.getComponent(IEggStorage)

    @property
    def poller(self):
        return self.app.getComponent(IPoller)


class Home(PrefixHeaderMixin, resource.Resource):

    def __init__(self, root, local_items):
        resource.Resource.__init__(self)
        self.root = root
        self.local_items = local_items
        self.prefix_header = root.prefix_header

    def render_GET(self, txrequest):
        vars = {
            'base_path': self.get_base_path(txrequest),
        }
        s = """\
<html>
<head><title>Scrapyd</title></head>
<body>
<h1>Scrapyd</h1>
<ul>
<li><a href="%(base_path)s/jobs">Jobs</a></li>
"""
        if self.local_items:
            s += '<li><a href="%(base_path)s/items/">Items</a></li>\n'
        s += """\
<li><a href="%(base_path)s/logs/">Logs</a></li>
<li><a href="https://scrapyd.readthedocs.io/en/latest/">Documentation</a></li>
</ul>
""" % vars
        if self.root.scheduler.list_projects():
            s += '<p>Available projects:<p>\n<ul>\n'
            for project_name in sorted(self.root.scheduler.list_projects()):
                s += f'<li>{project_name}</li>\n'
            s += '</ul>\n'
        else:
            s += '<p>No projects available.</p>\n'
        s += """
<h2>How to schedule a spider?</h2>

<p>To schedule a spider you need to use the API (this web UI is only for
monitoring)</p>

<p>Example using <a href="https://curl.se/">curl</a>:</p>
<p><code>curl http://localhost:6800/schedule.json -d project=default -d spider=somespider</code></p>

<p>For more information about the API, see the
<a href="https://scrapyd.readthedocs.io/en/latest/">Scrapyd documentation</a></p>
</body>
</html>
"""
        txrequest.setHeader('Content-Type', 'text/html; charset=utf-8')
        s = (s % vars).encode('utf8')
        txrequest.setHeader('Content-Length', str(len(s)))
        return s


def microsec_trunc(timelike):
    if hasattr(timelike, 'microsecond'):
        ms = timelike.microsecond
    else:
        ms = timelike.microseconds
    return timelike - timedelta(microseconds=ms)


class Jobs(PrefixHeaderMixin, resource.Resource):

    def __init__(self, root, local_items):
        resource.Resource.__init__(self)
        self.root = root
        self.local_items = local_items
        self.prefix_header = root.prefix_header

    cancel_button = """
    <form method="post" action="{base_path}/cancel.json">
    <input type="hidden" name="project" value="{project}"/>
    <input type="hidden" name="job" value="{jobid}"/>
    <input type="submit" style="float: left;" value="Cancel"/>
    </form>
    """.format

    header_cols = [
        'Project', 'Spider',
        'Job', 'PID',
        'Start', 'Runtime', 'Finish',
        'Log', 'Items',
        'Cancel',
    ]

    def gen_css(self):
        css = [
            '#jobs>thead td {text-align: center; font-weight: bold}',
            '#jobs>tbody>tr:first-child {background-color: #eee}',
        ]
        if not self.local_items:
            col_idx = self.header_cols.index('Items') + 1
            css.append('#jobs>*>tr>*:nth-child(%d) {display: none}' % col_idx)
        if b'cancel.json' not in self.root.children:
            col_idx = self.header_cols.index('Cancel') + 1
            css.append('#jobs>*>tr>*:nth-child(%d) {display: none}' % col_idx)
        return '\n'.join(css)

    def prep_row(self, cells):
        if not isinstance(cells, dict):
            assert len(cells) == len(self.header_cols)
        else:
            cells = [cells.get(k) for k in self.header_cols]
        cells = ['<td>%s</td>' % ('' if c is None else c) for c in cells]
        return '<tr>%s</tr>' % ''.join(cells)

    def prep_doc(self):
        return (
            '<html>'
            '<head>'
            '<title>Scrapyd</title>'
            '<style type="text/css">' + self.gen_css() + '</style>'
            '</head>'
            '<body><h1>Jobs</h1>'
            '<p><a href="./">Go up</a></p>'
            + self.prep_table() +
            '</body>'
            '</html>'
        )

    def prep_table(self):
        return (
            '<table id="jobs" border="1">'
            '<thead>' + self.prep_row(self.header_cols) + '</thead>'
            '<tbody>'
            + '<tr><th colspan="%d">Pending</th></tr>' % len(self.header_cols)
            + self.prep_tab_pending() +
            '</tbody>'
            '<tbody>'
            + '<tr><th colspan="%d">Running</th></tr>' % len(self.header_cols)
            + self.prep_tab_running() +
            '</tbody>'
            '<tbody>'
            + '<tr><th colspan="%d">Finished</th></tr>' % len(self.header_cols)
            + self.prep_tab_finished() +
            '</tbody>'
            '</table>'
        )

    def prep_tab_pending(self):
        return '\n'.join(
            self.prep_row({
                "Project": project,
                "Spider": m['name'],
                "Job": m['_job'],
                "Cancel": self.cancel_button(project=project, jobid=m['_job'], base_path=self.base_path),
            })
            for project, queue in self.root.poller.queues.items()
            for m in queue.list()
        )

    def prep_tab_running(self):
        return '\n'.join(
            self.prep_row({
                "Project": p.project,
                "Spider": p.spider,
                "Job": p.job,
                "PID": p.pid,
                "Start": microsec_trunc(p.start_time),
                "Runtime": microsec_trunc(datetime.now() - p.start_time),
                "Log": f'<a href="{self.base_path}{job_log_url(p)}">Log</a>',
                "Items": f'<a href="{self.base_path}{job_items_url(p)}">Items</a>',
                "Cancel": self.cancel_button(project=p.project, jobid=p.job, base_path=self.base_path),
            })
            for p in self.root.launcher.processes.values()
        )

    def prep_tab_finished(self):
        return '\n'.join(
            self.prep_row({
                "Project": p.project,
                "Spider": p.spider,
                "Job": p.job,
                "Start": microsec_trunc(p.start_time),
                "Runtime": microsec_trunc(p.end_time - p.start_time),
                "Finish": microsec_trunc(p.end_time),
                "Log": f'<a href="{self.base_path}{job_log_url(p)}">Log</a>',
                "Items": f'<a href="{self.base_path}{job_items_url(p)}">Items</a>',
            })
            for p in self.root.launcher.finished
        )

    def render(self, txrequest):
        self.base_path = self.get_base_path(txrequest)
        doc = self.prep_doc()
        txrequest.setHeader('Content-Type', 'text/html; charset=utf-8')
        doc = doc.encode('utf-8')
        txrequest.setHeader('Content-Length', str(len(doc)))
        return doc



import pkgutil
import sys

import pytest

from scrapyd.scripts.scrapyd_run import main

__version__ = pkgutil.get_data(__package__, '../VERSION').decode('ascii').strip()


def test_print_version(capsys, monkeypatch):
    monkeypatch.setattr(sys, 'argv', ['scrapyd', 'junk', '--version', 'junk'])
    main()

    assert capsys.readouterr().out == f"Scrapyd {__version__}\n"


def test_print_v(capsys, monkeypatch):
    monkeypatch.setattr(sys, 'argv', ['scrapyd', 'junk', '-v', 'junk'])
    main()

    assert capsys.readouterr().out == f"Scrapyd {__version__}\n"


def test_twisted_options(capsys, monkeypatch):
    """
    Test that the twisted options are correctly parsed.
    """
    monkeypatch.setattr(sys, 'argv', ['scrapyd', '--help'])

    with pytest.raises(SystemExit):
        main()

    assert 'twistd' in capsys.readouterr().out



class TestWebsite:
    def test_render_jobs(self, txrequest, site_no_egg):
        content = site_no_egg.children[b'jobs'].render(txrequest)
        expect_headers = {
            b'Content-Type': [b'text/html; charset=utf-8'],
            b'Content-Length': [b'643'],
        }
        if site_no_egg.local_items:
            expect_headers[b'Content-Length'] = [b'601']

        headers = dict(txrequest.responseHeaders.getAllRawHeaders())

        assert headers == expect_headers
        assert content.decode().startswith(
            '<html><head><title>Scrapyd</title><style type="text/css">'
            '#jobs>thead td {text-align: center; font-weight'
        )
        if site_no_egg.local_items:
            assert b'display: none' not in content
        else:
            assert b'display: none' in content

    def test_render_home(self, txrequest, site_no_egg):
        content = site_no_egg.children[b''].render_GET(txrequest)
        expect_headers = {
            b'Content-Type': [b'text/html; charset=utf-8'],
            b'Content-Length': [b'714'],
        }
        if site_no_egg.local_items:
            expect_headers[b'Content-Length'] = [b'751']

        headers = dict(txrequest.responseHeaders.getAllRawHeaders())

        assert headers == expect_headers
        assert b'Available projects' in content
        if site_no_egg.local_items:
            assert b'Items' in content
        else:
            assert b'Items' not in content



import sys

from scrapy.utils.misc import load_object
from twisted.application.internet import TCPServer, TimerService
from twisted.application.service import Application
from twisted.cred.portal import Portal
from twisted.python import log
from twisted.web import server
from twisted.web.guard import BasicCredentialFactory, HTTPAuthSessionWrapper

from scrapyd.basicauth import PublicHTMLRealm, StringCredentialsChecker
from scrapyd.environ import Environment
from scrapyd.interfaces import IEggStorage, IEnvironment, IJobStorage, IPoller, ISpiderScheduler
from scrapyd.poller import QueuePoller
from scrapyd.scheduler import SpiderScheduler


def create_wrapped_resource(webcls, config, app):
    username = config.get('username', '')
    password = config.get('password', '')
    if ':' in username:
        sys.exit("The `username` option contains illegal character ':', "
                 "check and update the configuration file of Scrapyd")
    resource = webcls(config, app)
    if username and password:
        log.msg("Basic authentication enabled")
        portal = Portal(PublicHTMLRealm(resource),
                        [StringCredentialsChecker(username, password)])
        credential_factory = BasicCredentialFactory("Auth")
        return HTTPAuthSessionWrapper(portal, [credential_factory])
    else:
        log.msg("Basic authentication disabled as either `username` or `password` is unset")
        return resource


def application(config):
    app = Application("Scrapyd")
    http_port = config.getint('http_port', 6800)
    bind_address = config.get('bind_address', '127.0.0.1')
    poll_interval = config.getfloat('poll_interval', 5)

    poller = QueuePoller(config)
    scheduler = SpiderScheduler(config)
    environment = Environment(config)

    app.setComponent(IPoller, poller)
    app.setComponent(ISpiderScheduler, scheduler)
    app.setComponent(IEnvironment, environment)

    jspath = config.get('jobstorage', 'scrapyd.jobstorage.MemoryJobStorage')
    jscls = load_object(jspath)
    jobstorage = jscls(config)
    app.setComponent(IJobStorage, jobstorage)
    eggstorage = config.get('eggstorage', 'scrapyd.eggstorage.FilesystemEggStorage')
    eggstoragecls = load_object(eggstorage)
    app.setComponent(IEggStorage, eggstoragecls(config))

    laupath = config.get('launcher', 'scrapyd.launcher.Launcher')
    laucls = load_object(laupath)
    launcher = laucls(config, app)

    timer = TimerService(poll_interval, poller.poll)

    webpath = config.get('webroot', 'scrapyd.website.Root')
    webcls = load_object(webpath)
    resource = create_wrapped_resource(webcls, config, app)
    webservice = TCPServer(http_port, server.Site(resource), interface=bind_address)
    log.msg(format="Scrapyd web console available at http://%(bind_address)s:%(http_port)s/",
            bind_address=bind_address, http_port=http_port)

    launcher.setServiceParent(app)
    timer.setServiceParent(app)
    webservice.setServiceParent(app)

    return app



#!/usr/bin/env python

import pkgutil
import sys
from os.path import dirname, join

from twisted.scripts.twistd import run

import scrapyd


def main():
    if len(sys.argv) > 1 and '-v' in sys.argv[1:] or '--version' in sys.argv[1:]:
        __version__ = pkgutil.get_data(__package__, '../VERSION').decode('ascii').strip()
        print(f'Scrapyd {__version__}')
    else:
        sys.argv[1:1] = ['-n', '-y', join(dirname(scrapyd.__file__), 'txapp.py')]
        run()


if __name__ == '__main__':
    main()



from urllib.parse import urljoin

import requests


def req(method, path, auth=None, **kwargs):
    url = urljoin("http://127.0.0.1:6800", path)

    for badauth in (None, ("baduser", "badpass")):
        response = getattr(requests, method)(url, auth=badauth, **kwargs)

        assert response.status_code == 401, response.status_code
        assert response.text == "Unauthorized"

    response = getattr(requests, method)(url, auth=("hello12345", "67890world"), **kwargs)

    assert response.status_code == 200, response.status_code

    return response



import os.path

from setuptools import setup

with open('README.rst') as f:
    long_description = f.read()

with open(os.path.join(os.path.dirname(__file__), 'scrapyd', 'VERSION')) as f:
    version = f.read().strip()

setup(
    name='samge-fork-scrapyd',
    version=version,
    author='samge',
    author_email='samgeapp@gmail.com',
    url='https://github.com/Samge0/scrapyd',
    description='A service for running Scrapy spiders, with an HTTP API',
    license='BSD',
    packages=['scrapyd'],
    long_description=long_description,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    # The scrapyd command requires the txapp.py to be decompressed. #49
    zip_safe=False,
    install_requires=[
        'packaging',
        'twisted>=17.9',
        'scrapy>=2.0.0',
        'setuptools',
        'w3lib',
        'zope.interface',
    ],
    extras_require={
        'test': [
            'pytest',
            'pytest-cov',
            'requests',
        ],
        'docs': [
            'furo',
            'sphinx',
            'sphinx-autobuild',
        ],
    },
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: No Input/Output (Daemon)',
        'Topic :: Internet :: WWW/HTTP',
    ],
    entry_points={
        'console_scripts': [
            'scrapyd = scrapyd.scripts.scrapyd_run:main'
        ]
    }
)



from integration_tests import req


def assert_webservice(method, path, expected, **kwargs):
    response = req(method, path, **kwargs)
    json = response.json()
    json.pop("node_name")

    assert json == expected


def test_listjobs():
    assert_webservice(
        "get",
        "/listjobs.json",
        {"status": "ok", "pending": [], "running": [], "finished": []},
    )


def test_listprojects():
    assert_webservice(
        "get",
        "/listprojects.json",
        {"status": "ok", "projects": []},
    )


def test_delproject_nonexistent():
    assert_webservice(
        "post",
        "/delproject.json",
        {"status": "error", "message": "[Errno 2] No such file or directory: 'eggs/nonexistent'"},
        data={"project": "nonexistent"},
    )
